----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

realCruiseControl = {};
realCruiseControl.modDirectory  = g_currentModDirectory;
local modDescFile = loadXMLFile("modDesc", realCruiseControl.modDirectory .. "modDesc.xml");
realCruiseControl.title = getXMLString(modDescFile, "modDesc.title.en");
realCruiseControl.author = getXMLString(modDescFile, "modDesc.author");
realCruiseControl.version = getXMLString(modDescFile, "modDesc.version");
realCruiseControl.key = 0;
delete(modDescFile);

function realCruiseControl.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)
		and SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function realCruiseControl.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", realCruiseControl);
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", realCruiseControl);
end;

function realCruiseControl:onLoad()
	self.realCruiseControlEvents = {};
	self.realCruiseControlEventIds = {};
end;

function realCruiseControl:setCruiseControl()
    local spec = self.spec_drivable;
    if spec.cruiseControl ~= nil then
		local speed = self:getLastSpeed();
		if speed < 1 then return; end;
		math.floor(speed);
		self:setCruiseControlMaxSpeed(speed);
		spec.cruiseControl.speed = speed;
        spec.cruiseControl.state = Drivable.CRUISECONTROL_STATE_ACTIVE;
        if not self.isServer then
            g_client:getServerConnection():sendEvent(SetCruiseControlStateEvent:new(self, Drivable.CRUISECONTROL_STATE_ACTIVE));
        else
            local owner = self:getOwner();
            if owner ~= nil then
                owner:sendEvent(SetCruiseControlStateEvent:new(self, Drivable.CRUISECONTROL_STATE_ACTIVE));
            end;
        end;
        if spec.toggleCruiseControlEvent ~= nil then
            local text = g_i18n:getText("action_deactivateCruiseControl");
            g_inputBinding:setActionEventText(spec.toggleCruiseControlEvent, text);
        end;
    end;
end;

function realCruiseControl:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
		self:clearActionEventsTable(self.realCruiseControlEvents);
		self.realCruiseControlEventIds = {};
        if isActiveForInputIgnoreSelection then
			local _, actionEventId = self:addActionEvent(self.realCruiseControlEvents, 'realCruiseControl_setKey', self, realCruiseControl.setCruiseControl, false, true, false, true, nil);
			g_inputBinding:setActionEventTextVisibility(actionEventId, true);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_HIGH);
			table.insert(self.realCruiseControlEventIds, actionEventId);
			if actionEventId == "" then
				_, actionEventId = self:addActionEvent(self.realCruiseControlEvents, 'realCruiseControl_setKey', self, realCruiseControl.setCruiseControl, false, true, false, true, nil);
				g_inputBinding:setActionEventTextVisibility(actionEventId, true);
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_HIGH);
				table.insert(self.realCruiseControlEventIds, actionEventId);
			end;
		end;
	end;
	realCruiseControl:setKey();
end;

function realCruiseControl:updateVehiclePhysics(superFunc, axisForward, axisSide, doHandbrake, dt)
	local spec = self.spec_drivable;
	axisForward = axisForward;
	axisSide = self:getSteeringDirection() * axisSide;
	local acceleration = 0;
	if self:getIsMotorStarted() and self:getMotorStartTime() <= g_currentMission.time then
		acceleration = axisForward;
		if math.abs(acceleration) > 0 and not Input.isKeyPressed(realCruiseControl.key) then
			self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF);
		end;
		if spec.cruiseControl.state ~= Drivable.CRUISECONTROL_STATE_OFF then
			acceleration = 1.0;
		end;
	end;
	if not self:getCanMotorRun() then
		acceleration = 0;
		if self:getIsMotorStarted() then
			self:stopMotor();
		end;
	end;
	-- only update steering if a player is in the vehicle
	if self.getIsControlled ~= nil and self:getIsControlled() then
		local targetRotatedTime = 0;
		if self.maxRotTime ~= nil and self.minRotTime ~= nil then
			if axisSide < 0 then
				-- 0 to maxRotTime
				targetRotatedTime = math.min(-self.maxRotTime * axisSide, self.maxRotTime);
			else
				-- 0 to minRotTime
				targetRotatedTime = math.max(self.minRotTime * axisSide, self.minRotTime);
			end;
		end;
		self.rotatedTime = targetRotatedTime;
	end;
	if self.firstTimeRun then
		if self.spec_wheels ~= nil then
			WheelsUtil.updateWheelsPhysics(self, dt, self.lastSpeedReal*self.movingDirection, acceleration, doHandbrake, g_currentMission.missionInfo.stopAndGoBraking);
		end;
	end;
	return acceleration;
end;

function realCruiseControl:setKey()
	local xmlFile = loadXMLFile("inputBinding", getUserProfileAppPath().. "inputBinding.xml")
	local i = 0;
	while true do
		local objKey = string.format("inputBinding.actionBinding(%d)", i);
		local inputKey = getXMLString(xmlFile, objKey.."#action")
		if inputKey == nil then break; end;
		if string.lower(inputKey) == "realcruisecontrol_setkey" then
			realCruiseControl.key = Input[getXMLString(xmlFile, objKey.. ".binding#input")];
			break;
		end;
		i = i + 1;
	end;
end;

Drivable.updateVehiclePhysics = Utils.overwrittenFunction(Drivable.updateVehiclePhysics, realCruiseControl.updateVehiclePhysics);
print("Load mod: "..realCruiseControl.title.." : v"..realCruiseControl.version.." by "..realCruiseControl.author);